@@ -174,7 +174,7 @@ class ConsoleMetricExporter(MetricExporter):
 
 
 class MetricReader(ABC):
-    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-branches,broad-exception-raised
     """
     Base class for all metric readers
 
@@ -247,27 +247,27 @@ class MetricReader(ABC):
                 if typ is Counter:
                     self._instrument_class_temporality[_Counter] = temporality
                 elif typ is UpDownCounter:
-                    self._instrument_class_temporality[
-                        _UpDownCounter
-                    ] = temporality
+                    self._instrument_class_temporality[_UpDownCounter] = (
+                        temporality
+                    )
                 elif typ is Histogram:
-                    self._instrument_class_temporality[
-                        _Histogram
-                    ] = temporality
+                    self._instrument_class_temporality[_Histogram] = (
+                        temporality
+                    )
                 elif typ is Gauge:
                     self._instrument_class_temporality[_Gauge] = temporality
                 elif typ is ObservableCounter:
-                    self._instrument_class_temporality[
-                        _ObservableCounter
-                    ] = temporality
+                    self._instrument_class_temporality[_ObservableCounter] = (
+                        temporality
+                    )
                 elif typ is ObservableUpDownCounter:
                     self._instrument_class_temporality[
                         _ObservableUpDownCounter
                     ] = temporality
                 elif typ is ObservableGauge:
-                    self._instrument_class_temporality[
-                        _ObservableGauge
-                    ] = temporality
+                    self._instrument_class_temporality[_ObservableGauge] = (
+                        temporality
+                    )
                 else:
                     raise Exception(f"Invalid instrument class found {typ}")
 
@@ -287,27 +287,27 @@ class MetricReader(ABC):
                 if typ is Counter:
                     self._instrument_class_aggregation[_Counter] = aggregation
                 elif typ is UpDownCounter:
-                    self._instrument_class_aggregation[
-                        _UpDownCounter
-                    ] = aggregation
+                    self._instrument_class_aggregation[_UpDownCounter] = (
+                        aggregation
+                    )
                 elif typ is Histogram:
-                    self._instrument_class_aggregation[
-                        _Histogram
-                    ] = aggregation
+                    self._instrument_class_aggregation[_Histogram] = (
+                        aggregation
+                    )
                 elif typ is Gauge:
                     self._instrument_class_aggregation[_Gauge] = aggregation
                 elif typ is ObservableCounter:
-                    self._instrument_class_aggregation[
-                        _ObservableCounter
-                    ] = aggregation
+                    self._instrument_class_aggregation[_ObservableCounter] = (
+                        aggregation
+                    )
                 elif typ is ObservableUpDownCounter:
                     self._instrument_class_aggregation[
                         _ObservableUpDownCounter
                     ] = aggregation
                 elif typ is ObservableGauge:
-                    self._instrument_class_aggregation[
-                        _ObservableGauge
-                    ] = aggregation
+                    self._instrument_class_aggregation[_ObservableGauge] = (
+                        aggregation
+                    )
                 else:
                     raise Exception(f"Invalid instrument class found {typ}")
 
@@ -398,13 +398,13 @@ class InMemoryMetricReader(MetricReader):
             preferred_aggregation=preferred_aggregation,
         )
         self._lock = RLock()
-        self._metrics_data: (
-            "opentelemetry.sdk.metrics.export.MetricsData"
-        ) = None
+        self._metrics_data: "opentelemetry.sdk.metrics.export.MetricsData" = (
+            None
+        )
 
     def get_metrics_data(
         self,
-    ) -> ("opentelemetry.sdk.metrics.export.MetricsData"):
+    ) -> "opentelemetry.sdk.metrics.export.MetricsData":
         """Reads and returns current metrics from the SDK"""
         with self._lock:
             self.collect()
@@ -496,7 +496,7 @@ class PeriodicExportingMetricReader(MetricReader):
         elif self._export_interval_millis <= 0:
             raise ValueError(
                 f"interval value {self._export_interval_millis} is invalid \
-                and needs to be larger than zero and lower than infinity."
+                and needs to be larger than zero."
             )
 
     def _at_fork_reinit(self):
@@ -519,7 +519,13 @@ class PeriodicExportingMetricReader(MetricReader):
                     exc_info=True,
                 )
         # one last collection below before shutting down completely
-        self.collect(timeout_millis=self._export_interval_millis)
+        try:
+            self.collect(timeout_millis=self._export_interval_millis)
+        except MetricsTimeoutError:
+            _logger.warning(
+                "Metric collection timed out.",
+                exc_info=True,
+            )
 
     def _receive_metrics(
         self,
@@ -529,12 +535,13 @@ class PeriodicExportingMetricReader(MetricReader):
     ) -> None:
 
         token = attach(set_value(_SUPPRESS_INSTRUMENTATION_KEY, True))
+        # pylint: disable=broad-exception-caught,invalid-name
         try:
             with self._export_lock:
                 self._exporter.export(
                     metrics_data, timeout_millis=timeout_millis
                 )
-        except Exception as e:  # pylint: disable=broad-except,invalid-name
+        except Exception as e:
             _logger.exception("Exception while exporting metrics %s", str(e))
         detach(token)
 
@@ -551,9 +558,7 @@ class PeriodicExportingMetricReader(MetricReader):
 
         self._shutdown_event.set()
         if self._daemon_thread:
-            self._daemon_thread.join(
-                timeout=(deadline_ns - time_ns()) / 10**9
-            )
+            self._daemon_thread.join(timeout=(deadline_ns - time_ns()) / 10**9)
         self._exporter.shutdown(timeout=(deadline_ns - time_ns()) / 10**6)
 
     def force_flush(self, timeout_millis: float = 10_000) -> bool:
