@@ -100,18 +100,18 @@ class SynchronousMeasurementConsumer(MeasurementConsumer):
             metric_reader_storage = self._reader_storages[metric_reader]
             # for now, just use the defaults
             callback_options = CallbackOptions()
-            deadline_ns = time_ns() + timeout_millis * 10**6
+            deadline_ns = time_ns() + (timeout_millis * 1e6)
 
-            default_timeout_millis = 10000 * 10**6
+            default_timeout_ns = 10000 * 1e6
 
             for async_instrument in self._async_instruments:
 
                 remaining_time = deadline_ns - time_ns()
 
-                if remaining_time < default_timeout_millis:
+                if remaining_time < default_timeout_ns:
 
                     callback_options = CallbackOptions(
-                        timeout_millis=remaining_time
+                        timeout_millis=remaining_time / 1e6
                     )
 
                 measurements = async_instrument.callback(callback_options)
