@@ -59,10 +59,13 @@ import abc
 import concurrent.futures
 import logging
 import os
+import platform
 import sys
 import typing
 from json import dumps
 from os import environ
+from types import ModuleType
+from typing import List, MutableMapping, Optional, cast
 from urllib import parse
 
 from opentelemetry.attributes import BoundedAttributes
@@ -75,13 +78,17 @@ from opentelemetry.semconv.resource import ResourceAttributes
 from opentelemetry.util._importlib_metadata import entry_points, version
 from opentelemetry.util.types import AttributeValue
 
+psutil: Optional[ModuleType] = None
+
 try:
-    import psutil
+    import psutil as psutil_module
+
+    pustil = psutil_module
 except ImportError:
-    psutil = None
+    pass
 
 LabelValue = AttributeValue
-Attributes = typing.Dict[str, LabelValue]
+Attributes = typing.Mapping[str, LabelValue]
 logger = logging.getLogger(__name__)
 
 CLOUD_PROVIDER = ResourceAttributes.CLOUD_PROVIDER
@@ -119,8 +126,9 @@ KUBERNETES_JOB_UID = ResourceAttributes.K8S_JOB_UID
 KUBERNETES_JOB_NAME = ResourceAttributes.K8S_JOB_NAME
 KUBERNETES_CRON_JOB_UID = ResourceAttributes.K8S_CRONJOB_UID
 KUBERNETES_CRON_JOB_NAME = ResourceAttributes.K8S_CRONJOB_NAME
-OS_TYPE = ResourceAttributes.OS_TYPE
 OS_DESCRIPTION = ResourceAttributes.OS_DESCRIPTION
+OS_TYPE = ResourceAttributes.OS_TYPE
+OS_VERSION = ResourceAttributes.OS_VERSION
 PROCESS_PID = ResourceAttributes.PROCESS_PID
 PROCESS_PARENT_PID = ResourceAttributes.PROCESS_PARENT_PID
 PROCESS_EXECUTABLE_NAME = ResourceAttributes.PROCESS_EXECUTABLE_NAME
@@ -141,12 +149,15 @@ TELEMETRY_SDK_VERSION = ResourceAttributes.TELEMETRY_SDK_VERSION
 TELEMETRY_AUTO_VERSION = ResourceAttributes.TELEMETRY_AUTO_VERSION
 TELEMETRY_SDK_LANGUAGE = ResourceAttributes.TELEMETRY_SDK_LANGUAGE
 
-_OPENTELEMETRY_SDK_VERSION = version("opentelemetry-sdk")
+_OPENTELEMETRY_SDK_VERSION: str = version("opentelemetry-sdk")
 
 
 class Resource:
     """A Resource is an immutable representation of the entity producing telemetry as Attributes."""
 
+    _attributes: BoundedAttributes
+    _schema_url: str
+
     def __init__(
         self, attributes: Attributes, schema_url: typing.Optional[str] = None
     ):
@@ -173,17 +184,19 @@ class Resource:
         if not attributes:
             attributes = {}
 
-        resource_detectors = []
+        otel_experimental_resource_detectors = {"otel"}.union(
+            {
+                otel_experimental_resource_detector.strip()
+                for otel_experimental_resource_detector in environ.get(
+                    OTEL_EXPERIMENTAL_RESOURCE_DETECTORS, ""
+                ).split(",")
+                if otel_experimental_resource_detector
+            }
+        )
 
-        resource = _DEFAULT_RESOURCE
-
-        otel_experimental_resource_detectors = environ.get(
-            OTEL_EXPERIMENTAL_RESOURCE_DETECTORS, "otel"
-        ).split(",")
-
-        if "otel" not in otel_experimental_resource_detectors:
-            otel_experimental_resource_detectors.append("otel")
+        resource_detectors: List[ResourceDetector] = []
 
+        resource_detector: str
         for resource_detector in otel_experimental_resource_detectors:
             resource_detectors.append(
                 next(
@@ -191,19 +204,19 @@ class Resource:
                         entry_points(
                             group="opentelemetry_resource_detector",
                             name=resource_detector.strip(),
-                        )
+                        )  # type: ignore
                     )
                 ).load()()
             )
-
         resource = get_aggregated_resources(
             resource_detectors, _DEFAULT_RESOURCE
         ).merge(Resource(attributes, schema_url))
 
         if not resource.attributes.get(SERVICE_NAME, None):
             default_service_name = "unknown_service"
-            process_executable_name = resource.attributes.get(
-                PROCESS_EXECUTABLE_NAME, None
+            process_executable_name = cast(
+                Optional[str],
+                resource.attributes.get(PROCESS_EXECUTABLE_NAME, None),
             )
             if process_executable_name:
                 default_service_name += ":" + process_executable_name
@@ -241,8 +254,8 @@ class Resource:
         Returns:
             The newly-created Resource.
         """
-        merged_attributes = self.attributes.copy()
-        merged_attributes.update(other.attributes)
+        merged_attributes = self.attributes.copy()  # type: ignore
+        merged_attributes.update(other.attributes)  # type: ignore
 
         if self.schema_url == "":
             schema_url = other.schema_url
@@ -257,8 +270,7 @@ class Resource:
                 other.schema_url,
             )
             return self
-
-        return Resource(merged_attributes, schema_url)
+        return Resource(merged_attributes, schema_url)  # type: ignore
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, Resource):
@@ -268,15 +280,18 @@ class Resource:
             and self._schema_url == other._schema_url
         )
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(
-            f"{dumps(self._attributes.copy(), sort_keys=True)}|{self._schema_url}"
+            f"{dumps(self._attributes.copy(), sort_keys=True)}|{self._schema_url}"  # type: ignore
         )
 
-    def to_json(self, indent=4) -> str:
+    def to_json(self, indent: int = 4) -> str:
+        attributes: MutableMapping[str, AttributeValue] = dict(
+            self._attributes
+        )
         return dumps(
             {
-                "attributes": dict(self._attributes),
+                "attributes": attributes,  # type: ignore
                 "schema_url": self._schema_url,
             },
             indent=indent,
@@ -294,7 +309,7 @@ _DEFAULT_RESOURCE = Resource(
 
 
 class ResourceDetector(abc.ABC):
-    def __init__(self, raise_on_error=False):
+    def __init__(self, raise_on_error: bool = False) -> None:
         self.raise_on_error = raise_on_error
 
     @abc.abstractmethod
@@ -335,10 +350,12 @@ class ProcessResourceDetector(ResourceDetector):
         _runtime_version = ".".join(
             map(
                 str,
-                sys.version_info[:3]
-                if sys.version_info.releaselevel == "final"
-                and not sys.version_info.serial
-                else sys.version_info,
+                (
+                    sys.version_info[:3]
+                    if sys.version_info.releaselevel == "final"
+                    and not sys.version_info.serial
+                    else sys.version_info
+                ),
             )
         )
         _process_pid = os.getpid()
@@ -346,7 +363,7 @@ class ProcessResourceDetector(ResourceDetector):
         _process_executable_path = os.path.dirname(_process_executable_name)
         _process_command = sys.argv[0]
         _process_command_line = " ".join(sys.argv)
-        _process_command_args = sys.argv[1:]
+        _process_command_args = sys.argv
         resource_info = {
             PROCESS_RUNTIME_DESCRIPTION: sys.version,
             PROCESS_RUNTIME_NAME: sys.implementation.name,
@@ -363,16 +380,101 @@ class ProcessResourceDetector(ResourceDetector):
             resource_info[PROCESS_PARENT_PID] = os.getppid()
 
         if psutil is not None:
-            process = psutil.Process()
-            resource_info[PROCESS_OWNER] = process.username()
+            process: psutil_module.Process = psutil.Process()
+            username = process.username()
+            resource_info[PROCESS_OWNER] = username
 
-        return Resource(resource_info)
+        return Resource(resource_info)  # type: ignore
+
+
+class OsResourceDetector(ResourceDetector):
+    """Detect os resources based on `Operating System conventions <https://opentelemetry.io/docs/specs/semconv/resource/os/>`_."""
+
+    def detect(self) -> "Resource":
+        """Returns a resource with with ``os.type`` and ``os.version``.
+
+        Python's platform library
+        ~~~~~~~~~~~~~~~~~~~~~~~~~
+
+        To grab this information, Python's ``platform`` does not return what a
+        user might expect it to. Below is a breakdown of its return values in
+        different operating systems.
+
+        .. code-block:: python
+            :caption: Linux
+
+            >>> platform.system()
+            'Linux'
+            >>> platform.release()
+            '6.5.0-35-generic'
+            >>> platform.version()
+            '#35~22.04.1-Ubuntu SMP PREEMPT_DYNAMIC Tue May  7 09:00:52 UTC 2'
+
+        .. code-block:: python
+            :caption: MacOS
+
+            >>> platform.system()
+            'Darwin'
+            >>> platform.release()
+            '23.0.0'
+            >>> platform.version()
+            'Darwin Kernel Version 23.0.0: Fri Sep 15 14:42:57 PDT 2023; root:xnu-10002.1.13~1/RELEASE_ARM64_T8112'
+
+        .. code-block:: python
+            :caption: Windows
+
+            >>> platform.system()
+            'Windows'
+            >>> platform.release()
+            '2022Server'
+            >>> platform.version()
+            '10.0.20348'
+
+        .. code-block:: python
+            :caption: FreeBSD
+
+            >>> platform.system()
+            'FreeBSD'
+            >>> platform.release()
+            '14.1-RELEASE'
+            >>> platform.version()
+            'FreeBSD 14.1-RELEASE releng/14.1-n267679-10e31f0946d8 GENERIC'
+
+        .. code-block:: python
+            :caption: Solaris
+
+            >>> platform.system()
+            'SunOS'
+            >>> platform.release()
+            '5.11'
+            >>> platform.version()
+            '11.4.0.15.0'
+
+        """
+
+        os_type = platform.system().lower()
+        os_version = platform.release()
+
+        # See docstring
+        if os_type == "windows":
+            os_version = platform.version()
+        # Align SunOS with conventions
+        elif os_type == "sunos":
+            os_type = "solaris"
+            os_version = platform.version()
+
+        return Resource(
+            {
+                OS_TYPE: os_type,
+                OS_VERSION: os_version,
+            }
+        )
 
 
 def get_aggregated_resources(
     detectors: typing.List["ResourceDetector"],
     initial_resource: typing.Optional[Resource] = None,
-    timeout=5,
+    timeout: int = 5,
 ) -> "Resource":
     """Retrieves resources from detectors in the order that they were passed
 
@@ -398,7 +500,7 @@ def get_aggregated_resources(
                     detector,
                     timeout,
                 )
-            # pylint: disable=broad-except
+            # pylint: disable=broad-exception-caught
             except Exception as ex:
                 if detector.raise_on_error:
                     raise ex
