@@ -15,12 +15,14 @@
 # pylint: disable=protected-access
 
 import unittest
-from unittest.mock import patch
+from unittest.mock import Mock, patch
 
 from opentelemetry.sdk._logs import LoggerProvider
 from opentelemetry.sdk._logs._internal import (
+    NoOpLogger,
     SynchronousMultiLogRecordProcessor,
 )
+from opentelemetry.sdk.environment_variables import OTEL_SDK_DISABLED
 from opentelemetry.sdk.resources import Resource
 
 
@@ -53,6 +55,7 @@ class TestLoggerProvider(unittest.TestCase):
             "name",
             version="version",
             schema_url="schema_url",
+            attributes={"key": "value"},
         )
 
         self.assertEqual(logger._instrumentation_scope.name, "name")
@@ -60,6 +63,14 @@ class TestLoggerProvider(unittest.TestCase):
         self.assertEqual(
             logger._instrumentation_scope.schema_url, "schema_url"
         )
+        self.assertEqual(
+            logger._instrumentation_scope.attributes, {"key": "value"}
+        )
+
+    @patch.dict("os.environ", {OTEL_SDK_DISABLED: "true"})
+    def test_get_logger_with_sdk_disabled(self):
+        logger = LoggerProvider().get_logger(Mock())
+        self.assertIsInstance(logger, NoOpLogger)
 
     @patch.object(Resource, "create")
     def test_logger_provider_init(self, resource_patch):
