@@ -12,10 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# pylint: disable=protected-access,too-many-lines,invalid-name
+# pylint: disable=consider-using-enumerate,no-self-use,too-many-public-methods
+
+from inspect import currentframe
 from itertools import permutations
 from logging import WARNING
 from math import ldexp
-from sys import float_info
+from random import Random, randrange
+from sys import float_info, maxsize
 from types import MethodType
 from unittest.mock import Mock, patch
 
@@ -37,6 +42,9 @@ from opentelemetry.sdk.metrics._internal.exponential_histogram.mapping.logarithm
     LogarithmMapping,
 )
 from opentelemetry.sdk.metrics._internal.measurement import Measurement
+from opentelemetry.sdk.metrics._internal.point import (
+    ExponentialHistogramDataPoint,
+)
 from opentelemetry.sdk.metrics.view import (
     ExponentialBucketHistogramAggregation,
 )
@@ -66,8 +74,8 @@ def swap(
 ):
 
     for attribute in [
-        "_positive",
-        "_negative",
+        "_value_positive",
+        "_value_negative",
         "_sum",
         "_count",
         "_zero_count",
@@ -130,14 +138,18 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         self.assertEqual(a._mapping.scale, b._mapping.scale)
 
-        self.assertEqual(len(a._positive), len(b._positive))
-        self.assertEqual(len(a._negative), len(b._negative))
+        self.assertEqual(len(a._value_positive), len(b._value_positive))
+        self.assertEqual(len(a._value_negative), len(b._value_negative))
 
-        for index in range(len(a._positive)):
-            self.assertEqual(a._positive[index], b._positive[index])
+        for index in range(len(a._value_positive)):
+            self.assertEqual(
+                a._value_positive[index], b._value_positive[index]
+            )
 
-        for index in range(len(a._negative)):
-            self.assertEqual(a._negative[index], b._negative[index])
+        for index in range(len(a._value_negative)):
+            self.assertEqual(
+                a._value_negative[index], b._value_negative[index]
+            )
 
     def test_alternating_growth_0(self):
         """
@@ -154,7 +166,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         # agg is an instance of github.com/lightstep/otel-launcher-go/lightstep/sdk/metric/aggregator/histogram/structure.Histogram[float64]
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=4)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=4
+            )
         )
 
         exponential_histogram_aggregation.aggregate(Measurement(2, Mock()))
@@ -162,11 +176,12 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         exponential_histogram_aggregation.aggregate(Measurement(1, Mock()))
 
         self.assertEqual(
-            exponential_histogram_aggregation._positive.offset, -1
+            exponential_histogram_aggregation._value_positive.offset, -1
         )
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, 0)
         self.assertEqual(
-            get_counts(exponential_histogram_aggregation._positive), [1, 1, 1]
+            get_counts(exponential_histogram_aggregation._value_positive),
+            [1, 1, 1],
         )
 
     def test_alternating_growth_1(self):
@@ -178,7 +193,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         """
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=4)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=4
+            )
         )
 
         exponential_histogram_aggregation.aggregate(Measurement(2, Mock()))
@@ -189,11 +206,12 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         exponential_histogram_aggregation.aggregate(Measurement(0.5, Mock()))
 
         self.assertEqual(
-            exponential_histogram_aggregation._positive.offset, -1
+            exponential_histogram_aggregation._value_positive.offset, -1
         )
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, -1)
         self.assertEqual(
-            get_counts(exponential_histogram_aggregation._positive), [2, 3, 1]
+            get_counts(exponential_histogram_aggregation._value_positive),
+            [2, 3, 1],
         )
 
     def test_permutations(self):
@@ -239,7 +257,10 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
                 exponential_histogram_aggregation = (
                     _ExponentialBucketHistogramAggregation(
-                        Mock(), Mock(), max_size=2
+                        Mock(),
+                        AggregationTemporality.DELTA,
+                        Mock(),
+                        max_size=2,
                     )
                 )
 
@@ -254,19 +275,19 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                     expected["scale"],
                 )
                 self.assertEqual(
-                    exponential_histogram_aggregation._positive.offset,
+                    exponential_histogram_aggregation._value_positive.offset,
                     expected["offset"],
                 )
                 self.assertEqual(
-                    len(exponential_histogram_aggregation._positive),
+                    len(exponential_histogram_aggregation._value_positive),
                     expected["len"],
                 )
                 self.assertEqual(
-                    exponential_histogram_aggregation._positive[0],
+                    exponential_histogram_aggregation._value_positive[0],
                     expected["at_0"],
                 )
                 self.assertEqual(
-                    exponential_histogram_aggregation._positive[1],
+                    exponential_histogram_aggregation._value_positive[1],
                     expected["at_1"],
                 )
 
@@ -285,7 +306,10 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
             exponential_histogram_aggregation = (
                 _ExponentialBucketHistogramAggregation(
-                    Mock(), Mock(), max_size=max_size
+                    Mock(),
+                    AggregationTemporality.DELTA,
+                    Mock(),
+                    max_size=max_size,
                 )
             )
 
@@ -310,7 +334,8 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                 init_scale, exponential_histogram_aggregation._mapping._scale
             )
             self.assertEqual(
-                offset, exponential_histogram_aggregation._positive.offset
+                offset,
+                exponential_histogram_aggregation._value_positive.offset,
             )
 
             exponential_histogram_aggregation.aggregate(
@@ -319,7 +344,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             sum_ += max_val
 
             self.assertNotEqual(
-                0, exponential_histogram_aggregation._positive[0]
+                0, exponential_histogram_aggregation._value_positive[0]
             )
 
             # The maximum-index filled bucket is at or
@@ -330,12 +355,15 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             total_count = 0
 
             for index in range(
-                len(exponential_histogram_aggregation._positive)
+                len(exponential_histogram_aggregation._value_positive)
             ):
-                total_count += exponential_histogram_aggregation._positive[
-                    index
-                ]
-                if exponential_histogram_aggregation._positive[index] != 0:
+                total_count += (
+                    exponential_histogram_aggregation._value_positive[index]
+                )
+                if (
+                    exponential_histogram_aggregation._value_positive[index]
+                    != 0
+                ):
                     max_fill = index
 
             # FIXME the corresponding Go code is
@@ -362,15 +390,15 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             index = mapping.map_to_index(min_val)
 
             self.assertEqual(
-                index, exponential_histogram_aggregation._positive.offset
+                index, exponential_histogram_aggregation._value_positive.offset
             )
 
             index = mapping.map_to_index(max_val)
 
             self.assertEqual(
                 index,
-                exponential_histogram_aggregation._positive.offset
-                + len(exponential_histogram_aggregation._positive)
+                exponential_histogram_aggregation._value_positive.offset
+                + len(exponential_histogram_aggregation._value_positive)
                 - 1,
             )
 
@@ -382,12 +410,12 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                 """
                 Increments a bucket
                 """
-
+                # pylint: disable=cell-var-from-loop
                 self._counts[bucket_index] += increment
 
             exponential_histogram_aggregation = (
                 _ExponentialBucketHistogramAggregation(
-                    Mock(), Mock(), max_size=256
+                    Mock(), AggregationTemporality.DELTA, Mock(), max_size=256
                 )
             )
 
@@ -398,15 +426,16 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             self.assertEqual(0, exponential_histogram_aggregation._sum)
             expect = 0
 
+            exponential_histogram_aggregation._value_positive = Buckets()
+
             for value in range(2, 257):
                 expect += value * increment
                 with patch.object(
-                    exponential_histogram_aggregation._positive,
+                    exponential_histogram_aggregation._value_positive,
                     "increment_bucket",
-                    # new=positive_mock
                     MethodType(
                         mock_increment,
-                        exponential_histogram_aggregation._positive,
+                        exponential_histogram_aggregation._value_positive,
                     ),
                 ):
                     exponential_histogram_aggregation.aggregate(
@@ -427,16 +456,16 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
             self.assertEqual(
                 256 - ((1 << scale) - 1),
-                len(exponential_histogram_aggregation._positive),
+                len(exponential_histogram_aggregation._value_positive),
             )
             self.assertEqual(
                 (1 << scale) - 1,
-                exponential_histogram_aggregation._positive.offset,
+                exponential_histogram_aggregation._value_positive.offset,
             )
 
             for index in range(0, 256):
                 self.assertLessEqual(
-                    exponential_histogram_aggregation._positive[index],
+                    exponential_histogram_aggregation._value_positive[index],
                     6 * increment,
                 )
 
@@ -444,12 +473,12 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         exponential_histogram_aggregation_0 = (
             _ExponentialBucketHistogramAggregation(
-                Mock(), Mock(), max_size=256
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=256
             )
         )
         exponential_histogram_aggregation_1 = (
             _ExponentialBucketHistogramAggregation(
-                Mock(), Mock(), max_size=256
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=256
             )
         )
 
@@ -482,36 +511,38 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         self.assertEqual(
             256 - ((1 << scale) - 1),
-            len(exponential_histogram_aggregation_1._positive),
+            len(exponential_histogram_aggregation_1._value_positive),
         )
         self.assertEqual(
             (1 << scale) - 1,
-            exponential_histogram_aggregation_1._positive.offset,
+            exponential_histogram_aggregation_1._value_positive.offset,
         )
 
         for index in range(0, 256):
             self.assertLessEqual(
-                exponential_histogram_aggregation_1._positive[index], 6
+                exponential_histogram_aggregation_1._value_positive[index], 6
             )
 
     def test_very_large_numbers(self):
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=2)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=2
+            )
         )
 
         def expect_balanced(count: int):
             self.assertEqual(
-                2, len(exponential_histogram_aggregation._positive)
+                2, len(exponential_histogram_aggregation._value_positive)
             )
             self.assertEqual(
-                -1, exponential_histogram_aggregation._positive.offset
+                -1, exponential_histogram_aggregation._value_positive.offset
             )
             self.assertEqual(
-                count, exponential_histogram_aggregation._positive[0]
+                count, exponential_histogram_aggregation._value_positive[0]
             )
             self.assertEqual(
-                count, exponential_histogram_aggregation._positive[1]
+                count, exponential_histogram_aggregation._value_positive[1]
             )
 
         exponential_histogram_aggregation.aggregate(
@@ -573,7 +604,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
     def test_full_range(self):
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=2)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=2
+            )
         )
 
         exponential_histogram_aggregation.aggregate(
@@ -595,18 +628,24 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         self.assertEqual(
             _ExponentialBucketHistogramAggregation._min_max_size,
-            len(exponential_histogram_aggregation._positive),
+            len(exponential_histogram_aggregation._value_positive),
         )
         self.assertEqual(
-            -1, exponential_histogram_aggregation._positive.offset
+            -1, exponential_histogram_aggregation._value_positive.offset
+        )
+        self.assertLessEqual(
+            exponential_histogram_aggregation._value_positive[0], 2
+        )
+        self.assertLessEqual(
+            exponential_histogram_aggregation._value_positive[1], 1
         )
-        self.assertLessEqual(exponential_histogram_aggregation._positive[0], 2)
-        self.assertLessEqual(exponential_histogram_aggregation._positive[1], 1)
 
     def test_aggregator_min_max(self):
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
         for value in [1, 3, 5, 7, 9]:
@@ -618,7 +657,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         self.assertEqual(9, exponential_histogram_aggregation._max)
 
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
         for value in [-1, -3, -5, -7, -9]:
@@ -632,21 +673,27 @@ class TestExponentialBucketHistogramAggregation(TestCase):
     def test_aggregator_copy_swap(self):
 
         exponential_histogram_aggregation_0 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
         for value in [1, 3, 5, 7, 9, -1, -3, -5]:
             exponential_histogram_aggregation_0.aggregate(
                 Measurement(value, Mock())
             )
         exponential_histogram_aggregation_1 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
         for value in [5, 4, 3, 2]:
             exponential_histogram_aggregation_1.aggregate(
                 Measurement(value, Mock())
             )
         exponential_histogram_aggregation_2 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
         swap(
@@ -654,8 +701,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             exponential_histogram_aggregation_1,
         )
 
-        exponential_histogram_aggregation_2._positive.__init__()
-        exponential_histogram_aggregation_2._negative.__init__()
+        # pylint: disable=unnecessary-dunder-call
+        exponential_histogram_aggregation_2._value_positive.__init__()
+        exponential_histogram_aggregation_2._value_negative.__init__()
         exponential_histogram_aggregation_2._sum = 0
         exponential_histogram_aggregation_2._count = 0
         exponential_histogram_aggregation_2._zero_count = 0
@@ -666,8 +714,8 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         )
 
         for attribute in [
-            "_positive",
-            "_negative",
+            "_value_positive",
+            "_value_negative",
             "_sum",
             "_count",
             "_zero_count",
@@ -689,7 +737,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
     def test_zero_count_by_increment(self):
 
         exponential_histogram_aggregation_0 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
         increment = 10
@@ -699,10 +749,11 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                 Measurement(0, Mock())
             )
         exponential_histogram_aggregation_1 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
-        # positive_mock = Mock(wraps=exponential_histogram_aggregation_1._positive)
         def mock_increment(self, bucket_index: int) -> None:
             """
             Increments a bucket
@@ -710,12 +761,14 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
             self._counts[bucket_index] += increment
 
+        exponential_histogram_aggregation_1._value_positive = Buckets()
+
         with patch.object(
-            exponential_histogram_aggregation_1._positive,
+            exponential_histogram_aggregation_1._value_positive,
             "increment_bucket",
-            # new=positive_mock
             MethodType(
-                mock_increment, exponential_histogram_aggregation_1._positive
+                mock_increment,
+                exponential_histogram_aggregation_1._value_positive,
             ),
         ):
             exponential_histogram_aggregation_1.aggregate(
@@ -732,7 +785,9 @@ class TestExponentialBucketHistogramAggregation(TestCase):
     def test_one_count_by_increment(self):
 
         exponential_histogram_aggregation_0 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
         increment = 10
@@ -742,10 +797,11 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                 Measurement(1, Mock())
             )
         exponential_histogram_aggregation_1 = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock())
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock()
+            )
         )
 
-        # positive_mock = Mock(wraps=exponential_histogram_aggregation_1._positive)
         def mock_increment(self, bucket_index: int) -> None:
             """
             Increments a bucket
@@ -753,12 +809,14 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
             self._counts[bucket_index] += increment
 
+        exponential_histogram_aggregation_1._value_positive = Buckets()
+
         with patch.object(
-            exponential_histogram_aggregation_1._positive,
+            exponential_histogram_aggregation_1._value_positive,
             "increment_bucket",
-            # new=positive_mock
             MethodType(
-                mock_increment, exponential_histogram_aggregation_1._positive
+                mock_increment,
+                exponential_histogram_aggregation_1._value_positive,
             ),
         ):
             exponential_histogram_aggregation_1.aggregate(
@@ -812,6 +870,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         exponential_histogram_aggregation = (
             _ExponentialBucketHistogramAggregation(
                 Mock(),
+                AggregationTemporality.DELTA,
                 Mock(),
                 max_size=_ExponentialBucketHistogramAggregation._min_max_size,
             )
@@ -825,7 +884,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         # This means the smallest max_scale is enough for the full range of the
         # normal floating point values.
         self.assertEqual(
-            len(exponential_histogram_aggregation._positive._counts),
+            len(exponential_histogram_aggregation._value_positive._counts),
             exponential_histogram_aggregation._min_max_size,
         )
 
@@ -836,6 +895,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         exponential_histogram_aggregation = (
             _ExponentialBucketHistogramAggregation(
                 Mock(),
+                AggregationTemporality.DELTA,
                 Mock(),
             )
         )
@@ -853,13 +913,15 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             AggregationTemporality.CUMULATIVE, 0
         )
 
-    def test_collect_results_cumulative(self):
+    def test_collect_results_cumulative(self) -> None:
         exponential_histogram_aggregation = (
             _ExponentialBucketHistogramAggregation(
                 Mock(),
+                AggregationTemporality.DELTA,
                 Mock(),
             )
         )
+        self.maxDiff = None
 
         self.assertEqual(exponential_histogram_aggregation._mapping._scale, 20)
 
@@ -884,7 +946,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         self.assertEqual(collection_0.zero_count, 0)
         self.assertEqual(
             collection_0.positive.bucket_counts,
-            [1, *[0] * 63, 1, *[0] * 31, 1, *[0] * 63],
+            [1, *[0] * 63, 1, *[0] * 63, 1, *[0] * 31],
         )
         self.assertEqual(collection_0.flags, 0)
         self.assertEqual(collection_0.min, 1)
@@ -911,7 +973,7 @@ class TestExponentialBucketHistogramAggregation(TestCase):
                 previous_count = count
                 count_counts.append([previous_count, 1])
 
-        self.assertEqual(collection_1.count, 5)
+        self.assertEqual(collection_1.count, 8)
         self.assertEqual(collection_1.sum, 16.645)
         self.assertEqual(collection_1.scale, 4)
         self.assertEqual(collection_1.zero_count, 0)
@@ -919,25 +981,89 @@ class TestExponentialBucketHistogramAggregation(TestCase):
         self.assertEqual(
             collection_1.positive.bucket_counts,
             [
-                1,
-                *[0] * 15,
-                1,
-                *[0] * 47,
-                1,
-                *[0] * 40,
                 1,
                 *[0] * 17,
                 1,
                 *[0] * 36,
+                1,
+                *[0] * 15,
+                2,
+                *[0] * 15,
+                1,
+                *[0] * 15,
+                1,
+                *[0] * 15,
+                1,
+                *[0] * 40,
             ],
         )
         self.assertEqual(collection_1.flags, 0)
         self.assertEqual(collection_1.min, 0.045)
         self.assertEqual(collection_1.max, 8)
 
+    def test_cumulative_aggregation_with_random_data(self) -> None:
+        histogram = _ExponentialBucketHistogramAggregation(
+            Mock(),
+            AggregationTemporality.DELTA,
+            Mock(),
+        )
+
+        def collect_and_validate(values, histogram) -> None:
+            result: ExponentialHistogramDataPoint = histogram.collect(
+                AggregationTemporality.CUMULATIVE, 0
+            )
+            buckets = result.positive.bucket_counts
+            scale = result.scale
+            index_start = result.positive.offset
+
+            for i in range(len(buckets)):
+                index = index_start + i
+                count = buckets[i]
+                lower_bound = 2 ** (index / (2**scale))
+                upper_bound = 2 ** ((index + 1) / (2**scale))
+                matches = 0
+                for value in values:
+                    # pylint: disable=chained-comparison
+                    if value > lower_bound and value <= upper_bound:
+                        matches += 1
+                assert (
+                    matches == count
+                ), f"index: {index}, count: {count}, scale: {scale}, lower_bound: {lower_bound}, upper_bound: {upper_bound}, matches: {matches}"
+
+            assert sum(buckets) + result.zero_count == len(values)
+            assert result.sum == sum(values)
+            assert result.count == len(values)
+            assert result.min == min(values)
+            assert result.max == max(values)
+            assert result.zero_count == len([v for v in values if v == 0])
+            assert scale >= 3
+
+        seed = randrange(maxsize)
+        # This test case is executed with random values every time. In order to
+        # run this test case with the same values used in a previous execution,
+        # check the value printed by that previous execution of this test case
+        # and use the same value for the seed variable in the line below.
+        # seed = 3373389994391084876
+
+        random_generator = Random(seed)
+        print(f"seed for {currentframe().f_code.co_name} is {seed}")
+
+        values = []
+        for i in range(2000):
+            # avoid both values being 0
+            value = random_generator.randint(0 if i else 1, 1000)
+            values.append(value)
+            histogram.aggregate(Measurement(value, Mock()))
+            if i % 20 == 0:
+                collect_and_validate(values, histogram)
+
+        collect_and_validate(values, histogram)
+
     def test_merge_collect_cumulative(self):
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=4)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=4
+            )
         )
 
         for value in [2, 4, 8, 16]:
@@ -946,16 +1072,21 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             )
 
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, 0)
-        self.assertEqual(exponential_histogram_aggregation._positive.offset, 0)
         self.assertEqual(
-            exponential_histogram_aggregation._positive.counts, [1, 1, 1, 1]
+            exponential_histogram_aggregation._value_positive.offset, 0
+        )
+        self.assertEqual(
+            exponential_histogram_aggregation._value_positive.counts,
+            [1, 1, 1, 1],
         )
 
-        result = exponential_histogram_aggregation.collect(
+        result_0 = exponential_histogram_aggregation.collect(
             AggregationTemporality.CUMULATIVE,
             0,
         )
 
+        self.assertEqual(result_0.scale, 0)
+
         for value in [1, 2, 4, 8]:
             exponential_histogram_aggregation.aggregate(
                 Measurement(1 / value, Mock())
@@ -963,10 +1094,11 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, 0)
         self.assertEqual(
-            exponential_histogram_aggregation._positive.offset, -4
+            exponential_histogram_aggregation._value_positive.offset, -4
         )
         self.assertEqual(
-            exponential_histogram_aggregation._positive.counts, [1, 1, 1, 1]
+            exponential_histogram_aggregation._value_positive.counts,
+            [1, 1, 1, 1],
         )
 
         result_1 = exponential_histogram_aggregation.collect(
@@ -974,11 +1106,13 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             0,
         )
 
-        self.assertEqual(result.scale, result_1.scale)
+        self.assertEqual(result_1.scale, -1)
 
     def test_merge_collect_delta(self):
         exponential_histogram_aggregation = (
-            _ExponentialBucketHistogramAggregation(Mock(), Mock(), max_size=4)
+            _ExponentialBucketHistogramAggregation(
+                Mock(), AggregationTemporality.DELTA, Mock(), max_size=4
+            )
         )
 
         for value in [2, 4, 8, 16]:
@@ -987,9 +1121,12 @@ class TestExponentialBucketHistogramAggregation(TestCase):
             )
 
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, 0)
-        self.assertEqual(exponential_histogram_aggregation._positive.offset, 0)
         self.assertEqual(
-            exponential_histogram_aggregation._positive.counts, [1, 1, 1, 1]
+            exponential_histogram_aggregation._value_positive.offset, 0
+        )
+        self.assertEqual(
+            exponential_histogram_aggregation._value_positive.counts,
+            [1, 1, 1, 1],
         )
 
         result = exponential_histogram_aggregation.collect(
@@ -1004,10 +1141,11 @@ class TestExponentialBucketHistogramAggregation(TestCase):
 
         self.assertEqual(exponential_histogram_aggregation._mapping.scale, 0)
         self.assertEqual(
-            exponential_histogram_aggregation._positive.offset, -4
+            exponential_histogram_aggregation._value_positive.offset, -4
         )
         self.assertEqual(
-            exponential_histogram_aggregation._positive.counts, [1, 1, 1, 1]
+            exponential_histogram_aggregation._value_positive.counts,
+            [1, 1, 1, 1],
         )
 
         result_1 = exponential_histogram_aggregation.collect(
