@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# pylint: disable=protected-access
+
 from math import inf
 from time import sleep
 from typing import Union
@@ -66,7 +68,7 @@ class TestSynchronousSumAggregation(TestCase):
         synchronous_sum_aggregation.aggregate(measurement(2))
         synchronous_sum_aggregation.aggregate(measurement(3))
 
-        self.assertEqual(synchronous_sum_aggregation._current_value, 6)
+        self.assertEqual(synchronous_sum_aggregation._value, 6)
 
         synchronous_sum_aggregation = _SumAggregation(
             Mock(), True, AggregationTemporality.DELTA, 0
@@ -76,7 +78,7 @@ class TestSynchronousSumAggregation(TestCase):
         synchronous_sum_aggregation.aggregate(measurement(-2))
         synchronous_sum_aggregation.aggregate(measurement(3))
 
-        self.assertEqual(synchronous_sum_aggregation._current_value, 2)
+        self.assertEqual(synchronous_sum_aggregation._value, 2)
 
     def test_aggregate_cumulative(self):
         """
@@ -91,7 +93,7 @@ class TestSynchronousSumAggregation(TestCase):
         synchronous_sum_aggregation.aggregate(measurement(2))
         synchronous_sum_aggregation.aggregate(measurement(3))
 
-        self.assertEqual(synchronous_sum_aggregation._current_value, 6)
+        self.assertEqual(synchronous_sum_aggregation._value, 6)
 
         synchronous_sum_aggregation = _SumAggregation(
             Mock(), True, AggregationTemporality.CUMULATIVE, 0
@@ -101,7 +103,7 @@ class TestSynchronousSumAggregation(TestCase):
         synchronous_sum_aggregation.aggregate(measurement(-2))
         synchronous_sum_aggregation.aggregate(measurement(3))
 
-        self.assertEqual(synchronous_sum_aggregation._current_value, 2)
+        self.assertEqual(synchronous_sum_aggregation._value, 2)
 
     def test_collect_delta(self):
         """
@@ -236,6 +238,8 @@ class TestLastValueAggregation(TestCase):
 
         self.assertEqual(first_number_data_point.value, 1)
 
+        self.assertIsNone(first_number_data_point.start_time_unix_nano)
+
         last_value_aggregation.aggregate(measurement(1))
 
         # CI fails the last assertion without this
@@ -249,6 +253,8 @@ class TestLastValueAggregation(TestCase):
 
         self.assertEqual(second_number_data_point.value, 1)
 
+        self.assertIsNone(second_number_data_point.start_time_unix_nano)
+
         self.assertGreater(
             second_number_data_point.time_unix_nano,
             first_number_data_point.time_unix_nano,
@@ -286,24 +292,16 @@ class TestExplicitBucketHistogramAggregation(TestCase):
         explicit_bucket_histogram_aggregation.aggregate(measurement(5))
 
         # The first bucket keeps count of values between (-inf, 0] (-1 and 0)
-        self.assertEqual(
-            explicit_bucket_histogram_aggregation._current_value[0], 2
-        )
+        self.assertEqual(explicit_bucket_histogram_aggregation._value[0], 2)
 
         # The second bucket keeps count of values between (0, 2] (1 and 2)
-        self.assertEqual(
-            explicit_bucket_histogram_aggregation._current_value[1], 2
-        )
+        self.assertEqual(explicit_bucket_histogram_aggregation._value[1], 2)
 
         # The third bucket keeps count of values between (2, 4] (3 and 4)
-        self.assertEqual(
-            explicit_bucket_histogram_aggregation._current_value[2], 2
-        )
+        self.assertEqual(explicit_bucket_histogram_aggregation._value[2], 2)
 
         # The fourth bucket keeps count of values between (4, inf) (3 and 4)
-        self.assertEqual(
-            explicit_bucket_histogram_aggregation._current_value[3], 1
-        )
+        self.assertEqual(explicit_bucket_histogram_aggregation._value[3], 1)
 
         histo = explicit_bucket_histogram_aggregation.collect(
             AggregationTemporality.CUMULATIVE, 1
